;; -*- no-byte-compile: t; lexical-binding: nil -*-
(define-package "lsp-mode" "20260702.527"
  "LSP mode."
  '((emacs         "29.1")
    (dash          "2.18.0")
    (f             "0.21.0")
    (ht            "2.3")
    (spinner       "1.7.3")
    (markdown-mode "2.3")
    (lv            "0")
    (eldoc         "1.11"))
  :url "https://github.com/emacs-lsp/lsp-mode"
  :commit "90bc0430f670c4da44e31c4fad1d2fb4a71dc909"
  :revdesc "90bc0430f670"
  :keywords '("languages"))
