;; The code was moved into lsp-mode.el. This file is kept only for backward compatibility. -*- lexical-binding: t -*-
(require 'lsp-mode)

;; (warn "Replace (require 'lsp) with (require 'lsp-mode)")

(provide 'lsp)

;;; lsp.el ends here
